# this file contains some common perl code for the other scripts in this directory

use File::Basename;
use Cwd qw(abs_path);

#$basedatadir = "/usr0/pfr/predpreydata/";

$scriptdir =  abs_path(dirname($0));
$basedir = abs_path("$scriptdir/..");
$worlddir = abs_path("$basedir/world");
$coachdir = abs_path("$basedir/coach");
$preddir = abs_path("$basedir/predator");
$preydir = abs_path("$basedir/prey");

sub bgexecute {
  my ($command) = (join ' ',@_);
  if ($pid = fork) {
    #parent
    print EXECUTELOGFH "$command\n";
    sleep 1;
    return $pid;
  } elsif (defined $pid) {
    #child
    #print "$$: about to run '$command'\n";
    `$command`;
    #print "$$: command finished '$command'\n";
    exit 0;
  } else {
    die "Could not execute process in the background: $!";
  }
}

sub run_exp
  {
    my ($sim_length, $predargs, $coachargs, $worldargs, $trialdir) = @_;

    $commargs = "--action_log_level 0 --logfile_dir $trialdir ";
    #$commargs = "--action_log_level 200 --logfile_dir $trialdir ";
    # make the agentdb
    $agentdbfile = "$trialdir/agentdb.xml";
    open (DBFH, ">$agentdbfile") || die "Could not open agentdb '$agentdbfile': $!";
    print DBFH <<EOF;
<?xml version="1.0" encoding="ISO-8859-1"?>

<agentdb
  xmlns:adb="http://spades-sim.sourceforge.net/agentdbxml.html"
  xmlns="http://spades-sim.sourceforge.net/agentdbxml.html"
  adb:version="0.92"
>

<!-- This file was generated by run_disable_exp.p -->
<agent_type_external adb:name="default">
  <timer>default</timer>
  <working_dir>.</working_dir>
  <exec_line>echo I am not an agent</exec_line>
</agent_type_external>
<agent_type_external adb:name="prey">
  <timer>default</timer>
  <working_dir>$preydir</working_dir>
  <exec_line>./prey $commargs</exec_line>
</agent_type_external>
<agent_type_external adb:name="pred">
  <timer>default</timer>
  <working_dir>$preddir</working_dir>
  <exec_line>./predator --file predator.conf $commargs $predargs --avg_reward_interval 5000 --avg_reward_fn $trialdir/rewards.data --qlearn_final_policy_fn $trialdir/pred_final.policy --qlearn_final_qtable_fn $trialdir/pred_final.qtable --qlearn_policy_change_fn $trialdir/pred_policy_change.data --qlearn_policy_eval_fn $trialdir/pred_policy_eval.data</exec_line>
</agent_type_external>
<agent_type_external adb:name="coach">
  <timer>default</timer>
  <working_dir>$coachdir</working_dir>
  <exec_line>coach --file coach.conf --run_mode online $commargs $coachargs --qlearn_final_policy_fn $trialdir/coach_final.policy --qlearn_final_qtable_fn $trialdir/coach_final.qtable</exec_line>
</agent_type_external>
</agentdb>
EOF
    close (DBFH);

    # run this thing!
    $cmdline = "$worlddir/predprey ";
    $cmdline .= "$worldargs ";
    $cmdline .= "--simulation_length $sim_length ";
    $cmdline .= "--agent_db_fn $agentdbfile ";
    $cmdline .= "--run_integrated_commserver on ";
    $cmdline .= $commargs;
#    $cmdline .= "--action_log_level 0 "; # I really insure that the engine doesn't log actions
    $cmdline .= " > $trialdir/worldoutput.log ";
    $enginepid = bgexecute($cmdline);

    # we no longer have to do this because we use an integrated comm server!
    #$cmdline = "$spadesdir/commserver/commserver --file $basedir/commserver.conf ";
    #$cmdline .= "--agent_db_fn $agentdbfile ";
    #$cmdline .= $commargs;
    #$cmdline .= "--action_log_level 0 "; # I really insure that the cs doesn't log actions
    #$cmdline .= " > $trialdir/csoutput.log ";
    #$cspid = bgexecute($cmdline);

    $ret = waitpid($enginepid, 0);

    die "Failure waiting for engine to die: $ret $?" if ($ret != $enginepid);

    #$ret = waitpid($cspid, 0);

    #die "Failure waiting for cs to die: $ret $?" if ($ret != $cspid);

    system "gzip $trialdir/monitor.log";
  }
