#!/usr/local/bin/perl -w

# this script runs experiments with the agent learning 
# no disabled actions

$usage = "$0 <data dir> <num trials>";

@betas = qw(0.0 0.1 0.3 0.5 0.7 0.9 1.0);

#$sim_length =   10000;
#$sim_length =   50000;
#$sim_length =  500000;
#$sim_length = 1500000;
$sim_length = 7000000;

use vars qw($basedatadir $enginepid $cspid);

use Cwd qw(abs_path);
use File::Basename;
use File::Copy;

#die "please set SPADESHOME" if (not exists $ENV{"SPADESHOME"});
#$spadesdir = abs_path($ENV{"SPADESHOME"});

$scriptdir = abs_path(dirname($0));
do "$scriptdir/common.pl" || die "Could not use common.pl: $!";

$basedatadir = shift @ARGV;
$trials = shift @ARGV;
die $usage if (!defined($trials));

# set up the output directory
$expdir = "$basedatadir/run_beta_exp.$$";
mkdir ($expdir, 0777) || die "Could not make exp dir '$expdir': $!";

print "Using experiment dir: $expdir\n";

# see common.pl
open (EXECUTELOGFH, ">$expdir/execute.log") || die "Could not open execute log: $!";

print EXECUTELOGFH "Run on ".`date`."\n";

# copy some conf files
copy "$basedir/world/predprey.conf", $expdir;
copy "$basedir/predator/predator.conf", $expdir;
copy "$basedir/coach/coach.conf", $expdir;
copy "$basedir/shared/shared.conf", $expdir;


foreach $beta (@betas) {

  $betadir = "$expdir/$beta";
  mkdir ($betadir, 0777) || die "Could not make beta dir '$betadir': $!";

  for ($cnt=0; $cnt < $trials; $cnt++) {
    # make the trial dir
    $trialdir = "$betadir/$cnt";
    mkdir ($trialdir, 0777) || die "Could not make trial dir '$trialdir': $!";

    run_exp($sim_length,
            "--act_mode qlearn_wcoach --qlearn_wcoach_coach_prob $beta",
            "--act_mode qlearn",
            "--file $expdir/predprey.conf --use_pred_coach on ",
            $trialdir);

    sleep(1);
  }
}

close (EXECUTELOGFH);

