/* A behavior to provide advice to execute a fixed policy */

#include "AdviseFixedBehavior.hpp"
#include "Logger.hpp"

using namespace spades;
using namespace spadesalib;

AdviseFixedBehavior::AdviseFixedBehavior(AgentCommInterface* pacs, const char* policy_fn)
  : Behavior(pacs), ppolicy(NULL)
{
  ppolicy = new Policy(policy_fn);
}

AdviseFixedBehavior::AdviseFixedBehavior(AgentCommInterface* pacs, Policy* p)
  : Behavior(pacs), ppolicy(p)
{
}

AdviseFixedBehavior::~AdviseFixedBehavior()
{
  if (ppolicy) delete ppolicy;
}

  
void
AdviseFixedBehavior::respondSense(Behavior::SenseType st)
{
  switch (st)
    {
    case ST_None:
      errorlog << "I should get ST_None" << ende;
      break;
    case ST_TimeNotify:
      //do nothing, we never request time notifies
      break;
    case ST_State:
      //do nothing, wait for an explicit request by the agent
      break;
    case ST_Comm:
      if (getCommType() == CT_RequestAction)
	{
	  sendAdvice((basic_move_t)ppolicy->getAction(getCurrState().flatVal()));
	}
      else
	{
	  errorlog << "I don't handle comms other than CT_RequestAction: " << getCommType() << ende;
	}
      break;
    default:
      errorlog << "I don't understand sense type: " << st << ende;
      break;
    }
}


bool
AdviseFixedBehavior::initialize()
{
  /* in the furture, we may want to calculate a rotation in order to handle multiple agents
     executing the "same" policy */
  return true;
}

