/* a virtual base class behavior for a coach to coach in a state/action framework
   The bandwidth restriction are the primary thing that this adds to the base q learner */

#include <iostream>
#include "BaseCoachSABehavior.hpp"
#include "State.hpp"
#include "Logger.hpp"
#include "CoachParam.hpp"

using namespace spades;
using namespace spadesalib;

BaseCoachSABehavior::BaseCoachSABehavior(AgentCommInterface* pacs,
					 float gamma,
					 explore_t exp, float alpha_explore, float boltzmann_constant,
					 const char* enable_map_fn)
  : BaseQLearnBehavior(pacs, gamma, exp, alpha_explore, boltzmann_constant),
    enable_map(enable_map_fn), psaar(NULL),
    final_advice_fn(),
    last_advice_send(SIMTIME_INVALID)
{
  qtable.disableAll();
  int count = qtable.enableSet(enable_map);
  actionlog(110) << "BaseCoachSABehavior: " << count << " actions enabled" << ende;
}

BaseCoachSABehavior::~BaseCoachSABehavior()
{
  if (!final_advice_fn.empty())
    {
      std::ofstream out(final_advice_fn.c_str());
      if (!out)
	errorlog << "Could not open final advice file '" << final_advice_fn << "'" << ende;
      out << *psaar;
    }

  if (psaar)
    delete psaar;
}


void
BaseCoachSABehavior::setSAAR(SAAdviceRepository* p)
{
  if (psaar)
    delete psaar;
  psaar = p;
}

void
BaseCoachSABehavior::respondSenseQLearn(Behavior::SenseType st)
{
  respondSenseSA(st);
  if (st == ST_State)
    {
      //update our bandwidth restriction information, and call getStateActionAdvice if applicable
      if (last_advice_send == SIMTIME_INVALID ||
	  pacs->getLatestTime() - last_advice_send > CoachParam::instance()->getSAAdviceInterval())
	{
	  vec_sa_advice_t vec;
	  getStateActionAdvice(CoachParam::instance()->getSAAdviceNumStates(), vec);

	  if (vec.size() > (unsigned)CoachParam::instance()->getSAAdviceNumStates())
	    {
	      errorlog << "Sub class wants to send too much advice! "
		       << vec.size() << '>' << CoachParam::instance()->getSAAdviceNumStates()
		       << ende;
	      vec.resize(CoachParam::instance()->getSAAdviceNumStates());
	    }

	  actionlog(50) << "Sending " << vec.size() << " pieces of advice" << ende;
	  sendSAAdvice(vec);
	  psaar->addAdvice(vec);
	  
	  last_advice_send = pacs->getLatestTime();
	}

      stateSenseUpdate();
    }
}

bool
BaseCoachSABehavior::initialize()
{
  if (!BaseQLearnBehavior::initialize())
    return false;
  return true;
}

const SAAdviceRepository*
BaseCoachSABehavior::getAdviceRepository() const
{
  if (psaar == NULL)
    errorlog << "BaseCoachCSABehavior: psaar is NULL" << ende;
  return psaar;
}
