/* -*- Mode: C++ -*- */

/* These are parameters for the coach */

#include "CoachParam.hpp"
#include "Logger.hpp"

using namespace spades;
using namespace std;

CoachParam* CoachParam::s_instance = NULL;

CoachParam::CoachParam (const int &argc, const char *const *argv)
  : PPSharedParam ()
{
  if (s_instance)
    errorlog << "Should only be one instance of CoachParam" << ende;
  s_instance = this;

  getOptions (argc, argv);
}

CoachParam::CoachParam ()
  : PPSharedParam ()
{
  if (s_instance)
    errorlog << "Should only be one instance of CoachParam" << ende;
  s_instance = this;
}

CoachParam::~CoachParam ()
{
  s_instance = NULL;
}

void
CoachParam::addAll2Maps ()
{
  PPSharedParam::addAll2Maps ();

  add2Maps("run_mode", &run_mode_string);

  add2Maps("act_mode", &act_mode_string);

  add2Maps("fixed_policy_fn", &fixed_policy_fn);

  add2Maps("fixed_actmap_1_fn", &fixed_actmap_1_fn);  
  add2Maps("fixed_actmap_2_fn", &fixed_actmap_2_fn);  
  add2Maps("fixed_actmap_3_fn", &fixed_actmap_3_fn);  
  
  add2Maps("sa_advise_states_per_cycle", &sa_advise_states_per_cycle);
  
  add2Maps("qlearn_explore", &qlearn_explore_string);
  add2Maps("qlearn_explore_prob", &qlearn_explore_prob);
  add2Maps("qlearn_boltzmann_constant", &qlearn_boltzmann_constant);
  add2Maps("qlearn_final_policy_fn", &qlearn_final_policy_fn);
  add2Maps("qlearn_final_qtable_fn", &qlearn_final_qtable_fn);
  add2Maps("qlearn_enable_map_fn", &qlearn_enable_map_fn);

  add2Maps("qlearn_cmpactmap_interval", &qlearn_cmpactmap_interval);
  add2Maps("qlearn_cmpactmap_actmap_fn", &qlearn_cmpactmap_actmap_fn);
  add2Maps("qlearn_cmpactmap_out_fn", &qlearn_cmpactmap_out_fn);

  add2Maps("qlearn_action_cmpactmap_interval", &qlearn_action_cmpactmap_interval);
  add2Maps("qlearn_action_cmpactmap_actmap_fn", &qlearn_action_cmpactmap_actmap_fn);
  add2Maps("qlearn_action_cmpactmap_out_fn", &qlearn_action_cmpactmap_out_fn);

  add2Maps("max_perc_for_qtable", &max_perc_for_qtable);

  add2Maps("mdp_fn", &mdp_fn);

  add2Maps("solve_mdp_out_fstem", &solve_mdp_out_fstem);

  add2Maps("extract_map_in_qtable_fn", &extract_map_in_qtable_fn);
  add2Maps("extract_map_out_fstem", &extract_map_out_fstem);

  add2Maps("cam_remove_one_opt", &cam_remove_one_opt);
  add2Maps("cam_fraction_non_opt", &cam_fraction_non_opt);
  add2Maps("cam_in_opt_fn", &cam_in_opt_fn);
  add2Maps("cam_in_nonopt_fn", &cam_in_nonopt_fn);
  add2Maps("cam_out_fn", &cam_out_fn);

  add2Maps("cmp_actmap_policy_fn", &cmp_actmap_policy_fn);
  add2Maps("cmp_actmap_actmap_fn", &cmp_actmap_actmap_fn);

  add2Maps("saadvice_interval", &saadvice_interval);
  add2Maps("saadvice_num_states", &saadvice_num_states);
  add2Maps("saadvice_final_advice_fn", &saadvice_final_advice_fn);

  add2Maps("saoptq_opt_qtable_fn", &saoptq_opt_qtable_fn);
  add2Maps("saoptq_no_advise_threshold", &saoptq_no_advise_threshold);
  add2Maps("saoptq_use_leftover", &saoptq_use_leftover);
}

void
CoachParam::setDefaultValues ()
{
  PPSharedParam::setDefaultValues ();

  run_mode_string = "online";
  run_mode = RM_Online;

  act_mode_string = "none";
  act_mode = AM_None;

  fixed_policy_fn = "";

  fixed_actmap_1_fn = "";  
  fixed_actmap_2_fn = "";
  fixed_actmap_3_fn = "";
  
  sa_advise_states_per_cycle = 3;
  
  qlearn_explore_string = "random";
  qlearn_explore = EXP_Random;
  qlearn_explore_prob = 0.1;
  qlearn_boltzmann_constant = 1.5;
  qlearn_final_policy_fn = "";  
  qlearn_final_qtable_fn = "";  
  qlearn_enable_map_fn = "";  

  qlearn_cmpactmap_interval = -1;
  qlearn_cmpactmap_actmap_fn = "tmp.actmap";
  qlearn_cmpactmap_out_fn = "qlearn_actmap_compare.out";

  qlearn_action_cmpactmap_interval = -1;
  qlearn_action_cmpactmap_actmap_fn = "tmp.actmap";
  qlearn_action_cmpactmap_out_fn = "qlearn_actmap_action_compare.out";

  max_perc_for_qtable = 0.9;
  
  mdp_fn = "";

  solve_mdp_out_fstem = "solved_mdp";

  extract_map_in_qtable_fn = "";
  extract_map_out_fstem = "extract";

  cam_remove_one_opt = false;
  cam_fraction_non_opt = 1.0;
  cam_in_opt_fn = "";
  cam_in_nonopt_fn = "";
  cam_out_fn = "cam.actmap";

  cmp_actmap_policy_fn = "";
  cmp_actmap_actmap_fn = "";

  saadvice_interval = 500;
  saadvice_num_states = 2;
  saadvice_final_advice_fn = "";

  saoptq_opt_qtable_fn = "";
  saoptq_no_advise_threshold = 0.1;
  saoptq_use_leftover = false;
}

void
CoachParam::postReadProcessing()
{
  PPSharedParam::postReadProcessing();

  if (run_mode_string == "none")
    run_mode = RM_None;
  else if (run_mode_string == "online")
    run_mode = RM_Online;
  else if (run_mode_string == "create_mdp")
    run_mode = RM_CreateMDP;
  else if (run_mode_string == "solve_mdp")
    run_mode = RM_SolveMDP;
  else if (run_mode_string == "extract_action_map")
    run_mode = RM_ExtractActionMap;
  else if (run_mode_string == "create_action_map")
    run_mode = RM_CreateActionMap;
  else if (run_mode_string == "pp_policy")
    run_mode = RM_PPPolicy;
  else if (run_mode_string == "cmp_action_map")
    run_mode = RM_CmpActionMap;
  else if (run_mode_string == "make_policy")
    run_mode = RM_MakePolicy;
  else 
    errorlog << "I didn't understand the run mode string! " << run_mode_string << ende;

  if (act_mode_string == "none")
    act_mode = AM_None;
  else if (act_mode_string == "null")
    act_mode = AM_Policy;
  else if (act_mode_string == "policy" || act_mode_string == "fixed")
    act_mode = AM_Policy;
  else if (act_mode_string == "actmap")
    act_mode = AM_ActMap;
  else if (act_mode_string == "qlearn")
    act_mode = AM_QLearn;
  else if (act_mode_string == "sa_policy")
    act_mode = AM_SAPolicy;
  else if (act_mode_string == "sa_random")
    act_mode = AM_SARandom;
  else if (act_mode_string == "sa_optq")
    act_mode = AM_SAOptQ;
  else
    errorlog << "I didn't understand the act mode string! " << act_mode_string << ende;

  const char* p = qlearn_explore_string.c_str();
  qlearn_explore = parseExplore(&p);
  if (qlearn_explore == EXP_None)
    errorlog << "Illegal value for explore: " << qlearn_explore_string << ende;

}
