/* coach parameters */

#ifndef _COACH_PARAM_H_
#define _COACH_PARAM_H_

#include "PPSharedParam.hpp"
#include "PPSharedTypes.hpp"
#include "QTable.hpp"

enum RunMode_t {
  RM_None,
  RM_Online,
  RM_CreateMDP,
  RM_SolveMDP,
  RM_ExtractActionMap,
  RM_CreateActionMap,
  RM_PPPolicy,
  RM_CmpActionMap,
  RM_MakePolicy
};

enum ActMode_t {
  AM_None,
  AM_Null,
  AM_Policy,
  AM_ActMap,
  AM_QLearn,
  AM_SAPolicy,
  AM_SARandom,
  AM_SAOptQ
};


class CoachParam : public PPSharedParam
{
public:

  CoachParam (const int &argc, const char *const *argv);
  CoachParam ();
  virtual ~CoachParam ();

  static const CoachParam* instance() { return s_instance; }
  
private:
  static CoachParam* s_instance;
    
protected:
  //virtual void writeConfig ( ostream& o );

  virtual void addAll2Maps ();
  virtual void setDefaultValues ();
  virtual void postReadProcessing();

public:

  RunMode_t getRunMode() const { return run_mode; }

  ActMode_t getActMode() const { return act_mode; }

  std::string getFixedPolicyFN() const { return fixed_policy_fn; }

  std::string getFixedActMap1FN() const { return fixed_actmap_1_fn; }
  std::string getFixedActMap2FN() const { return fixed_actmap_2_fn; }
  std::string getFixedActMap3FN() const { return fixed_actmap_3_fn; }
  
  int getSAAdviseStatesPerCycle() const { return sa_advise_states_per_cycle; }

  explore_t getQLearnExplore() const { return qlearn_explore; }
  double getQLearnExploreProb() const { return qlearn_explore_prob; }
  double getQLearnBoltzmannConstant() const { return qlearn_boltzmann_constant; }
  std::string getQLearnFinalPolicyFN() const { return qlearn_final_policy_fn; }
  std::string getQLearnFinalQTableFN() const { return qlearn_final_qtable_fn; }
  std::string getQLearnEnableMapFN() const { return qlearn_enable_map_fn; }

  int getQLearnCmpActmapInterval() const { return qlearn_cmpactmap_interval; }
  std::string getQLearnCmpActmapActmapFN() const { return qlearn_cmpactmap_actmap_fn; }
  std::string getQLearnCmpActmapOutFN() const { return qlearn_cmpactmap_out_fn; }

  int getQLearnActionCmpActmapInterval() const { return qlearn_action_cmpactmap_interval; }
  std::string getQLearnActionCmpActmapActmapFN() const { return qlearn_action_cmpactmap_actmap_fn; }
  std::string getQLearnActionCmpActmapOutFN() const { return qlearn_action_cmpactmap_out_fn; }

  double getMaxPercForQTable() const { return max_perc_for_qtable; }

  std::string getMDPFN() const { return mdp_fn; }

  std::string getSolveMDPOutFStem() const { return solve_mdp_out_fstem; }
  
  std::string getExtractMapInQTableFN() const { return extract_map_in_qtable_fn; }
  std::string getExtractMapOutFStem() const { return extract_map_out_fstem; }

  bool getCAMRemoveOneOpt() const { return cam_remove_one_opt; }
  double getCAMFractionNonOpt() const { return cam_fraction_non_opt; }
  std::string getCAMInOptFN() const { return cam_in_opt_fn; }
  std::string getCAMInNonoptFN() const { return cam_in_nonopt_fn; }
  std::string getCAMOutFN() const { return cam_out_fn; }

  std::string getCmpActmapPolicyFN() const { return cmp_actmap_policy_fn; }
  std::string getCmpActmapActmapFN() const { return cmp_actmap_actmap_fn; }

  int getSAAdviceInterval() const { return saadvice_interval; }
  int getSAAdviceNumStates() const { return saadvice_num_states; }
  std::string getSAAdviceFinalAdviceFN() const { return saadvice_final_advice_fn; }

  std::string getSAOptQOptQTableFN() const { return saoptq_opt_qtable_fn; }
  double getSAOptQNoAdviseThreshold() const { return saoptq_no_advise_threshold; }
  bool getSAOptQUseLeftover() const { return saoptq_use_leftover; }
  
protected:

  std::string run_mode_string;
  RunMode_t run_mode;
  
  ActMode_t act_mode;
  std::string act_mode_string;

  std::string fixed_policy_fn;

  std::string fixed_actmap_1_fn;
  std::string fixed_actmap_2_fn;
  std::string fixed_actmap_3_fn;
  
  int sa_advise_states_per_cycle;
  
  std::string qlearn_explore_string;
  explore_t qlearn_explore;
  double qlearn_explore_prob;
  double qlearn_boltzmann_constant;
  std::string qlearn_final_policy_fn;
  std::string qlearn_final_qtable_fn;
  std::string qlearn_enable_map_fn;

  int qlearn_cmpactmap_interval;
  std::string qlearn_cmpactmap_actmap_fn;
  std::string qlearn_cmpactmap_out_fn;
  
  int qlearn_action_cmpactmap_interval;
  std::string qlearn_action_cmpactmap_actmap_fn;
  std::string qlearn_action_cmpactmap_out_fn;
  
  double max_perc_for_qtable;

  std::string mdp_fn;

  std::string solve_mdp_out_fstem;

  std::string extract_map_in_qtable_fn;
  std::string extract_map_out_fstem;

  //cam is for CreateActionMap
  std::string cam_in_opt_fn;
  std::string cam_in_nonopt_fn;
  std::string cam_out_fn;
  bool cam_remove_one_opt; //if true, we remove one optimal action for the ouput
  double cam_fraction_non_opt; //fraction of non-opt actions to keep in output

  std::string cmp_actmap_policy_fn;
  std::string cmp_actmap_actmap_fn;
  
  int saadvice_interval;
  int saadvice_num_states;
  std::string saadvice_final_advice_fn;

  std::string saoptq_opt_qtable_fn;
  double saoptq_no_advise_threshold;
  bool saoptq_use_leftover;
  

};


#endif
