/* A behavior to provide advice to execute a fixed policy */

#include "SAAdviseFixedBehavior.hpp"
#include "random.hpp"
#include "Logger.hpp"
#include "CoachParam.hpp"

using namespace spades;
using namespace spadesalib;

SAAdviseFixedBehavior::SAAdviseFixedBehavior(AgentCommInterface* pacs, const char* policy_fn)
  : Behavior(pacs), ppolicy(NULL)
{
  ppolicy = new Policy(policy_fn);
}

SAAdviseFixedBehavior::SAAdviseFixedBehavior(AgentCommInterface* pacs, Policy* p)
  : Behavior(pacs), ppolicy(p)
{
}

SAAdviseFixedBehavior::~SAAdviseFixedBehavior()
{
  if (ppolicy) delete ppolicy;
}

  
void
SAAdviseFixedBehavior::respondSense(Behavior::SenseType st)
{
  switch (st)
    {
    case ST_None:
      errorlog << "I should get ST_None" << ende;
      break;
    case ST_TimeNotify:
      //do nothing, we never request time notifies
      break;
    case ST_State:
      adviseRandomStates(CoachParam::instance()->getSAAdviseStatesPerCycle());
      break;
    case ST_Comm:
      errorlog << "I don't handle comm from the agent: " << getCommType() << ende;
      break;
    default:
      errorlog << "I don't understand sense type: " << st << ende;
      break;
    }
}


bool
SAAdviseFixedBehavior::initialize()
{
  /* in the furture, we may want to calculate a rotation in order to handle multiple agents
     executing the "same" policy */
  return true;
}

void
SAAdviseFixedBehavior::adviseRandomStates(int num_states)
{
  vec_sa_advice_t v;
  
  for (int i=0; i < num_states; i++)
    {
      int stateval = int_random(State::getNumStates());
      SAAdvice sa(stateval, ppolicy->getAction(stateval));
      v.push_back(sa);
    }
  sendSAAdvice(v);
}


