/* -*- Mode: C++ -*- */

/* predator parameters */

#include "PredParam.hpp"
#include "Logger.hpp"

using namespace spades;
using namespace std;

PredParam* PredParam::s_instance = NULL;

PredParam::PredParam (const int &argc, const char *const *argv)
  : PPSharedParam ()
{
  if (s_instance)
    errorlog << "Should only be one instance of PPSharedParam" << ende;
  s_instance = this;

  getOptions (argc, argv);
}

PredParam::PredParam ()
  : PPSharedParam ()
{
  if (s_instance)
    errorlog << "Should only be one instance of PPSharedParam" << ende;
  s_instance = this;
}

PredParam::~PredParam ()
{
  s_instance = NULL;
}

void
PredParam::addAll2Maps ()
{
  PPSharedParam::addAll2Maps ();

  add2Maps("act_mode", &act_mode_string);

  add2Maps("fixed_policy_fn", &fixed_policy_fn);

  add2Maps("qlearn_explore", &qlearn_explore_string);
  add2Maps("qlearn_explore_prob", &qlearn_explore_prob);
  add2Maps("qlearn_boltzmann_constant", &qlearn_boltzmann_constant);
  add2Maps("qlearn_final_policy_fn", &qlearn_final_policy_fn);
  add2Maps("qlearn_final_qtable_fn", &qlearn_final_qtable_fn);
  add2Maps("qlearn_policy_change_fn", &qlearn_policy_change_fn);
  add2Maps("qlearn_policy_eval_fn", &qlearn_policy_eval_fn);
  add2Maps("qlearn_policy_eval_interval", &qlearn_policy_eval_interval);
  add2Maps("qlearn_policy_eval_length", &qlearn_policy_eval_length);
  add2Maps("qlearn_disable_map_fn", &qlearn_disable_map_fn);

  add2Maps("qlearn_wcoach_coach_prob", &qlearn_wcoach_coach_prob);
  add2Maps("qlearn_wcoach_random_on_advised_disable", &qlearn_wcoach_random_on_advised_disable);
  
  add2Maps("qlearn_csa_coach_prob", &qlearn_csa_coach_prob);
  add2Maps("saadvice_final_advice_fn", &saadvice_final_advice_fn);
}

void
PredParam::setDefaultValues ()
{
  PPSharedParam::setDefaultValues ();

  act_mode_string = "none";
  act_mode = AM_None;

  fixed_policy_fn = "";

  qlearn_explore_string = "random";
  qlearn_explore = EXP_Random;
  qlearn_explore_prob = 0.1;
  qlearn_boltzmann_constant = 1.5;
  qlearn_final_policy_fn = "";  
  qlearn_final_qtable_fn = "";  
  qlearn_policy_change_fn = "";  
  qlearn_policy_eval_fn = "";
  qlearn_policy_eval_interval = 100000;
  qlearn_policy_eval_length = 10000;
  qlearn_disable_map_fn = "";  

  qlearn_wcoach_coach_prob = .9;
  qlearn_wcoach_random_on_advised_disable = true;

  qlearn_csa_coach_prob = .9;
  saadvice_final_advice_fn = "";
}

void
PredParam::postReadProcessing()
{
  PPSharedParam::postReadProcessing();

  if (act_mode_string == "none")
    act_mode = AM_None;
  else if (act_mode_string == "random")
    act_mode = AM_Random;
  else if (act_mode_string == "policy")
    act_mode = AM_Policy;
  else if (act_mode_string == "qlearn")
    act_mode = AM_QLearn;
  else if (act_mode_string == "strict_coach_req")
    act_mode = AM_StrictCoachReq;
  else if (act_mode_string == "qlearn_wcoach")
    act_mode = AM_QLearnWCoach;
  else if (act_mode_string == "qlearn_csa_simple")
    act_mode = AM_QLearnCSASimple;
  else
    errorlog << "I didn't understand the act mode string! " << act_mode_string << ende;

  const char* p = qlearn_explore_string.c_str();
  qlearn_explore = parseExplore(&p);
  if (qlearn_explore == EXP_None)
    errorlog << "Illegal value for explore: " << qlearn_explore_string << ende;
}
