#include <stdio.h>
#include <unistd.h>
#include <string>
#include <signal.h>
#include "Logger.hpp"
#include "AgentCommInterface.hpp"
#include "utility.hpp"
#include "PreyTagFunction.hpp"
#include "PPSharedTypes.hpp"
#include "PPSharedParam.hpp"
#include "random.hpp"
#include "RandomBehavior.hpp"

using namespace spades;

/*****************************************************************************************/

void init_handler_term();
void init_handler_segv();

/*****************************************************************************************/

/*****************************************************************************************/

int main (int argc, char** argv)
{
  PPSharedParam *pParam;
  pParam = new PPSharedParam (argc, argv);

  PreyTagFunction *pTagFunc = new PreyTagFunction;

  spades::Logger::instance()->setTagFunction(pTagFunc);
  spades::Logger::instance()->setMaxActionLogLevel(pParam->getActionLogLevel());

  init_handler_term();
  init_handler_segv();

  spades::seedRandom(pParam->getRandomSeed());

  spadesalib::AgentCommInterface acs_interface;

  Behavior* beh = new RandomBehavior(&acs_interface);
  
  actionlog(10) << "Prey started" << ende;

  int res = beh->behaviorLoop();
  //int res = mainLoop(&acs_interface);

  actionlog(10) << "Client is exiting after " << res << " messages"
		<< ende;

  delete beh;
  
  Logger::removeInstance();

  delete pParam;

  return res;
}


/*****************************************************************************************/

void sigterm_handler(int) 
{
  exit(-1);
}

void init_handler_term() 
{
  struct sigaction sigact;
  sigact.sa_flags = 0;

  //This makes sure that the handler is only called once for each signal
  sigact.sa_flags |= SA_RESETHAND;
  
  sigemptyset(&sigact.sa_mask);
  sigact.sa_handler = sigterm_handler; 
  sigaction(SIGINT, &sigact, NULL);
  sigaction(SIGQUIT, &sigact, NULL);
  sigaction(SIGTERM, &sigact, NULL);
}

void sigsegv_handler(int) 
{
  errorlog << "Got a seg fault!" << ende;
  exit(-1);
}

void init_handler_segv() 
{
  struct sigaction sigact;
  sigact.sa_flags = 0;

  //This makes sure that the handler is only called once for each signal
  sigact.sa_flags |= SA_RESETHAND;
  
  sigemptyset(&sigact.sa_mask);
  sigact.sa_handler = sigsegv_handler; 
  sigaction(SIGSEGV, &sigact, NULL);
}
