/* This behavior is a virtual base class for qlearners built upon receiving coach messages
   about action advice that is state action pairs */

#include "BaseQLearnCSABehavior.hpp"
#include "State.hpp"
#include "Logger.hpp"

using namespace spades;
using namespace spadesalib;

BaseQLearnCSABehavior::BaseQLearnCSABehavior(spadesalib::AgentCommInterface* pacs,
					     float gamma,
					     explore_t exp, float alpha_explore, float boltzmann_constant)
  : BaseQLearnBehavior(pacs, gamma, exp, alpha_explore, boltzmann_constant), psaar(NULL)
{
}

BaseQLearnCSABehavior::~BaseQLearnCSABehavior()
{
  if (!final_advice_fn.empty())
    {
      std::ofstream out(final_advice_fn.c_str());
      if (!out)
	errorlog << "Could not open final advice file '" << final_advice_fn << "'" << ende;
      out << *psaar;
    }

  if (psaar)
    delete psaar;
}

void
BaseQLearnCSABehavior::setSAAR(SAAdviceRepository* p)
{
  if (psaar)
    delete psaar;
  psaar = p;
}

const SAAdviceRepository*
BaseQLearnCSABehavior::getSAAR() const
{
  if (psaar == NULL)
    errorlog << "BaseQLearnCSABehavior: psaar is NULL" << ende;
  return psaar;
}

void
BaseQLearnCSABehavior::respondSenseQLearn(Behavior::SenseType st)
{
  switch (st)
    {
    case ST_None:
      errorlog << "Shouldn't get ST_None here" << ende;
      break;
    case ST_TimeNotify:
      respondSenseCSA(st);
      break;
    case ST_State:
      updateQValue();
      //in case the child never sends an action, we default to none, since that's what happens
      setLastAction(BM_None); 
      respondSenseCSA(st);
      stateSenseUpdate();
      break;
    case ST_Comm:
      processComm();
      respondSenseCSA(st);
      break;
    default:
      errorlog << "I don't understand sense type: " << st << ende;
      break;
    }
}

bool
BaseQLearnCSABehavior::initialize()
{
  if (!BaseQLearnBehavior::initialize())
    return false;
  return true;
}

void
BaseQLearnCSABehavior::processComm()
{
  if (getCommType() == CT_StateActionAdvice)
    {
      actionlog(50) << "BaseQLearnCSABehavior: adding state/action advice, count = "
		    << getCommSAAdvice().size() << ende;
      psaar->addAdvice(getCommSAAdvice());
    }
}

