/* A behavior to execute a fixed policy */

#include "FixedPolicyBehavior.hpp"
#include "Logger.hpp"

using namespace spades;
using namespace spadesalib;

FixedPolicyBehavior::FixedPolicyBehavior(AgentCommInterface* pacs, const char* policy_fn)
  : Behavior(pacs), ppolicy(NULL)
{
  ppolicy = new Policy(policy_fn);
}

FixedPolicyBehavior::FixedPolicyBehavior(AgentCommInterface* pacs, Policy* p)
  : Behavior(pacs), ppolicy(p)
{
}

FixedPolicyBehavior::~FixedPolicyBehavior()
{
  if (ppolicy) delete ppolicy;
}

  
void
FixedPolicyBehavior::respondSense(Behavior::SenseType st)
{
  switch (st)
    {
    case ST_None:
      errorlog << "I should get ST_None" << ende;
      break;
    case ST_TimeNotify:
      //do nothing, we never request time notifies
      break;
    case ST_State:
      sendMove((basic_move_t)ppolicy->getAction(getCurrState().flatVal()));
      break;
    default:
      errorlog << "I don't understand sense type: " << st << ende;
      break;
    }
}


bool
FixedPolicyBehavior::initialize()
{
  /* in the furture, we may want to calculate a rotation in order to handle multiple agents
     executing the "same" policy */
  return true;
}

