/* -*- Mode: C++ -*- */

/* These are parameters shared between the predator and prey
   There is some duplication with the world parameters */

#include "PPSharedParam.hpp"
#include "Logger.hpp"
#include "PPSharedTypes.hpp"

using namespace spades;

PPSharedParam* PPSharedParam::s_instance = NULL;

PPSharedParam::PPSharedParam (const int &argc, const char *const *argv)
  : AgentLibraryParam ()
{
  if (s_instance)
    errorlog << "Should only be one instance of PPSharedParam" << ende;
  s_instance = this;

  getOptions (argc, argv);
}

PPSharedParam::PPSharedParam ()
{
  if (s_instance)
    errorlog << "Should only be one instance of PPSharedParam" << ende;
  s_instance = this;
}

PPSharedParam::~PPSharedParam ()
{
  s_instance = NULL;
}

void
PPSharedParam::addAll2Maps ()
{
  AgentLibraryParam::addAll2Maps ();

  add2Maps ("x_size", &x_size);
  add2Maps ("y_size", &y_size);

  add2Maps ("num_pred", &num_pred);
  add2Maps ("num_prey", &num_prey);

  //pfr 10/07/2004: now in the agent library params already
  //add2Maps ("random_seed", &random_seed);

  add2Maps("gamma", &gamma);

  add2Maps ("avg_reward_interval", &avg_reward_interval);
  add2Maps ("avg_reward_fn", &avg_reward_fn);

  add2Maps ("deterministic_update", &deterministic_update);

  add2Maps("forbid_all_q_updates", &forbid_all_q_updates);
  
  add2Maps ("reward", &reward);
  add2Maps ("per_step_reward", &per_step_reward);

  add2Maps ("saar_type", &saar_type_string);
  add2Maps ("saar_fifo_limit", &saar_fifo_limit);
}

void
PPSharedParam::setDefaultValues ()
{
  AgentLibraryParam::setDefaultValues ();

  x_size = 20;
  y_size = 20;

  num_pred = 0;
  num_prey = 0;

  //pfr 10/07/2004: now in the agent library params already
  //random_seed = -1;

  gamma = 0.9;

  avg_reward_interval = -1;
  avg_reward_fn = "rewards.out";

  deterministic_update = false;

  forbid_all_q_updates = false;
  
  reward = 10;
  per_step_reward = -1;

  saar_type = SAAR_None;
  saar_type_string = "none";
  saar_fifo_limit = 100;
}

void
PPSharedParam::postReadProcessing()
{
  AgentLibraryParam::postReadProcessing();

  total_size = x_size * y_size;

  Location::setSizes(x_size, y_size);

  if (saar_type_string == "none")
    saar_type = SAAR_None;
  else if (saar_type_string == "complete")
    saar_type = SAAR_Complete;
  else if (saar_type_string == "fifo")
    saar_type = SAAR_FIFO;
  else
    errorlog << "I didn't understand the saar type string: '" << saar_type_string << "'" << ende;
}
