/* These are parameters shared between the predator and prey
   There is some duplication with the world parameters */

#ifndef _PP_SHARED_PARAM_H_
#define _PP_SHARED_PARAM_H_

#include "AgentLibraryParam.hpp"

enum SAAR_t {
  SAAR_None,
  SAAR_Complete,
  SAAR_FIFO
};


class PPSharedParam : public spadesalib::AgentLibraryParam
{
public:

  PPSharedParam (const int &argc, const char *const *argv);
  PPSharedParam ();  
  virtual ~PPSharedParam ();

  static const PPSharedParam* instance() { return s_instance; }
  
private:
  static PPSharedParam* s_instance;
    
protected:
  //virtual void writeConfig ( ostream& o );

  virtual void addAll2Maps ();
  virtual void setDefaultValues ();
  virtual void postReadProcessing();

public:

  int getXSize() const { return x_size; }
  int getYSize() const { return y_size; }
  int getTotalSize() const { return total_size; } // = x_size * y_size
  
  int getNumPred() const { return num_pred; }
  int getNumPrey() const { return num_prey; }

  //pfr 10/07/2004: now in the agent library params already
  //int  getRandomSeed() const { return random_seed; }

  double getGamma() const { return gamma; }

  int getAvgRewardInterval() const { return avg_reward_interval; }
  std::string getAvgRewardFN() const { return avg_reward_fn; }

  bool getDeterministicUpdate() const { return deterministic_update; }

  bool getForbidAllQUpdates() const { return forbid_all_q_updates; }
  
  int getReward() const { return reward; }
  int getPerStepReward() const { return per_step_reward; }

  SAAR_t getSAARType() const { return saar_type; }
  int getSAARFIFOLimit() const { return saar_fifo_limit; }
  
protected:

  int x_size;
  int y_size;

  int total_size;
  
  int num_pred;
  int num_prey;

  //pfr 10/07/2004: now in the agent library params already
  //int  random_seed;

  double gamma;

  int avg_reward_interval;
  std::string avg_reward_fn;

  bool deterministic_update;

  bool forbid_all_q_updates;
  
  int reward;
  int per_step_reward;

  SAAR_t saar_type;
  std::string saar_type_string;
  int saar_fifo_limit;
};


#endif
