/* this file contains various types that are shared among the pred prey agents */

#include <cstring>
#include "PPSharedTypes.hpp"
#include "Logger.hpp"
#include "random.hpp"
#include "Logger.hpp"

using namespace std;
using namespace spades;

/***************************************************************************/

static const char* BASIC_MOVE_STRINGS[] = {"X", "N", "S", "E", "W"};

//ps is modified to point to the location one after the string
basic_move_t
parseBasicMove(const char** ps)
{
  for (int i=0; i<NUM_BASIC_MOVE_T; i++)
    {
      if (strncmp(*ps, BASIC_MOVE_STRINGS[i], strlen(BASIC_MOVE_STRINGS[i])) == 0)
	{
	  *ps += strlen(BASIC_MOVE_STRINGS[i]);
	  return (basic_move_t)i;
	}
    }
  return BM_None;
}

std::istream&
operator>>(std::istream& is, basic_move_t& m)
{
  char str[2];
  const char* p = str;
  char ch;
  
  while (isspace(ch = is.get())) ;
  str[0] = ch;
  str[1] = 0;

  m = parseBasicMove(&p);
  if (p == str)
    {
      is.setstate(ios::failbit);
      is.putback(ch);
      return is;
    }

  return is;
}



const char*
getBasicMoveString(const basic_move_t& m)
{
  if (m < 0 || m >= NUM_BASIC_MOVE_T)
    errorlog << "Trying to get basic move out of range! " << (int)m << ende;
  return BASIC_MOVE_STRINGS[(int)m];
}

std::ostream&
operator<<(std::ostream& o, const basic_move_t& m)
{
  o << getBasicMoveString(m);
  return o;
}

/***************************************************************************/

/* static elements */
int Location::s_x_size = 0;
int Location::s_y_size = 0;

void
Location::setRandom()
{
  x = int_random(s_x_size);
  y = int_random(s_y_size);
}

//the value is an flat enumeration of the possible states
unsigned int
Location::flatVal() const
{
  return y * s_y_size + x;
}

void
Location::setFromFlatVal(unsigned int flatval)
{
  y = flatval / s_y_size;
  x = flatval % s_y_size;
}



std::istream&
operator>>(std::istream& is, Location& l)
{
  char ch;  
  bool paren = false;
  
  while (isspace(ch = is.get())) ;
  if (ch != '(') is.putback(ch);
  else paren = true;  
  is >> l.x;
  if (!is) return is;
  while (isspace(ch = is.get())) ;
  if (ch != ',') is.putback(ch);
  is >> l.y;
  if (!is) return is;
  if (paren) {
    while (isspace(ch = is.get())) ;
    if (ch != ')') {
      is.putback(ch);
      is.setstate(ios::failbit);
    }
  }  
  return is;  
}

/* static */
void Location::setSizes(int x, int y)
{
  s_x_size = x;
  s_y_size = y;
}

