/* this file contains various types that are shared among the pred prey agents */

#ifndef _PP_SHARED_TYPES_H_
#define _PP_SHARED_TYPES_H_

#include <iostream>

enum basic_move_t {
  BM_None,
  BM_North,
  BM_South,
  BM_East,
  BM_West
};
const int NUM_BASIC_MOVE_T = 5;

//ps is modified to point to the location one after the string
basic_move_t parseBasicMove(const char** ps);
std::istream& operator>>(std::istream& i, basic_move_t& m);
const char* getBasicMoveString(const basic_move_t& m);

std::ostream& operator<<(std::ostream& o, const basic_move_t& m);
  

enum pp_agent_type_t {
  PPAT_None,
  PPAT_Pred,
  PPAT_Prey,
  PPAT_CoachPred,
  PPAT_CoachPrey
};

typedef int entityid_t;
const entityid_t PRED_COACH_ID = -1;
const entityid_t PREY_COACH_ID = -2;


class Location
{
public:
  Location() : x(0), y(0) {}
  Location(const Location& l) : x(l.x), y(l.y) {}
  Location(int x, int y) : x(x), y(y) {}
  ~Location() {}

  int getX() const { return x; }
  int getY() const { return y; }

  void setRandom();

  //the value is an flat enumeration of the possible states
  unsigned int flatVal() const;
  void setFromFlatVal(unsigned int flatval);
  
  const Location& operator=(const Location& l)
  { x = l.x; y = l.y; return *this; }

  bool operator==(const Location& l) const { return x == l.x && y == l.y; }

  friend std::ostream& operator<<(std::ostream& o, const Location& l)
  { o << "(" << l.x << "," << l.y << ")"; return o; }
  friend std::istream& operator>>(std::istream& i, Location& l);
  
  int x, y;

  /* static stuff */
public:
  static void setSizes(int x, int y);

private:
  static int s_x_size;
  static int s_y_size;
  
};


#endif
