/* a subclass of MDP to do some predprey specific stuff */

#include <math.h>
#include <vector>
#include "PredPreyMDP.hpp"
#include "PPSharedParam.hpp"
#include "Logger.hpp"
#include "State.hpp"

using namespace spades;
using namespace std;

PredPreyMDP::PredPreyMDP()
  : MDP()
{
}

PredPreyMDP::~PredPreyMDP()
{
}


void
PredPreyMDP::createFromCurrentState(bool wraparound, int reward)
{
  if (PPSharedParam::instance()->getNumPred() != 1)
    {
      errorlog << "MDP::createFromCurrentState: I can't handle these vals! "
	       << PPSharedParam::instance()->getNumPred() << ' ' << PPSharedParam::instance()->getNumPrey()
	       << ende;
      return;
    }
  
  num_states = State::getNumStates();
  num_actions = NUM_BASIC_MOVE_T;

  v_state_action_info.clear();
  v_state_action_info.resize(num_states * num_actions);

  float prey_act_prob = 1.0 / pow((float)NUM_BASIC_MOVE_T, (float)PPSharedParam::instance()->getNumPrey());

  for (unsigned state_idx = 0; state_idx < (unsigned)num_states; state_idx++)
    {
      actionlog(100) << "MDP::createFromState: working on " << state_idx << ende;
      State currstate(state_idx);

      /* we have to check if this is a capture because moves from a capture state are actually
	 undefined
	 We leave it with 0 transitions in order to reflect the lack of valid moves */
      if (currstate.isCapturePred(0))
	continue;
      
      for (basic_move_t predaction = (basic_move_t)0;
	   (int)predaction < NUM_BASIC_MOVE_T;
	   predaction = (basic_move_t) (predaction + 1))
	{
	  unsigned state_action_idx = state_idx * num_actions + (int)predaction;
	  State nextstate(currstate);
	  nextstate.applyBasicMovePred(predaction, 0, wraparound);

	  bool done = false;
	  vector<basic_move_t> vMoves;
	  vMoves.resize(PPSharedParam::instance()->getNumPrey(), (basic_move_t)0);
	  while (!done)
	    {
	      //apply the moves
	      State s(nextstate);
	      TranInfo info;
	      info.prob = prey_act_prob;
	      info.reward = PPSharedParam::instance()->getPerStepReward();
	      for (int preyidx = 0; preyidx < PPSharedParam::instance()->getNumPrey(); preyidx++)
		{
		  s.applyBasicMovePrey(vMoves[preyidx], preyidx, wraparound);
		  if (s.isCapture(0,preyidx))
		    info.reward += reward;
		}
	      info.nextstate = s.flatVal();

	      v_state_action_info[state_action_idx].push_back(info);

	      //set up the next vector of moves
	      for (int preyidx = 0; preyidx < PPSharedParam::instance()->getNumPrey(); preyidx++)
		{
		  vMoves[preyidx] = (basic_move_t)(vMoves[preyidx] + 1);
		  if (vMoves[preyidx] < NUM_BASIC_MOVE_T)
		    break;

		  if (preyidx == PPSharedParam::instance()->getNumPrey() - 1)
		    {
		      done = true;
		      break;
		    }
		  vMoves[preyidx] = (basic_move_t)0;
		}
	      
	    }
	}
    }

  actionlog(50) << "MDP::createFromState: done!" << ende;

  if (!verify(true))
    errorlog << "Created MDP failed verification!" << ende;
}


