/* A class decribing the state of the world */

/* This will use the global params in PPSharedParams about
   x_size, y_size, num_pred, num_prey */

#include "State.hpp"
#include "Logger.hpp"
#include "PPSharedParam.hpp"

using namespace std;
using namespace spades;

//returns x^y 
int int_pow(int x, int y)
{
  int res = 1;
  for (int i=0; i<y; i++)
    res *= x;
  return res;
}
unsigned int int_pow(unsigned int x, unsigned int y)
{
  unsigned int res = 1;
  for (unsigned int i=0; i<y; i++)
    res *= x;
  return res;
}

unsigned int
State::flatVal() const
{
  int position = PPSharedParam::instance()->getNumPred() + PPSharedParam::instance()->getNumPrey() - 1;
  int val = 0;
  
  for (unsigned int i = 0; i < (unsigned)PPSharedParam::instance()->getNumPred(); i++)
    {
      val += int_pow(PPSharedParam::instance()->getTotalSize(), position) *
	(i<vPredLoc.size() ? vPredLoc[i].flatVal() : 0);
      position--;
    }
  
  for (unsigned int i = 0; i < (unsigned)PPSharedParam::instance()->getNumPrey(); i++)
    {
      val += int_pow(PPSharedParam::instance()->getTotalSize(), position) *
	(i<vPreyLoc.size() ? vPreyLoc[i].flatVal() : 0);
      position--;
    }

  return val;
}

void
State::setFromFlatVal(unsigned flatval)
{
  vPredLoc.resize(PPSharedParam::instance()->getNumPred());
  vPreyLoc.resize(PPSharedParam::instance()->getNumPrey());

  int position = PPSharedParam::instance()->getNumPred() + PPSharedParam::instance()->getNumPrey() - 1;

  for (unsigned int i = 0; i < (unsigned)PPSharedParam::instance()->getNumPred(); i++)
    {
      unsigned thisval = flatval / int_pow(PPSharedParam::instance()->getTotalSize(), position);
      flatval %= int_pow(PPSharedParam::instance()->getTotalSize(), position);
      vPredLoc[i].setFromFlatVal(thisval);
      position--;
    }
  
  for (unsigned int i = 0; i < (unsigned)PPSharedParam::instance()->getNumPrey(); i++)
    {
      unsigned thisval = flatval / int_pow(PPSharedParam::instance()->getTotalSize(), position);
      flatval %= int_pow(PPSharedParam::instance()->getTotalSize(), position);
      vPreyLoc[i].setFromFlatVal(thisval);
      position--;
    }
}

void
State::applyBasicMove(std::vector<Location>& v, basic_move_t m, int idx, bool wraparound)
{
  Location pos = v[idx];

  switch (m)
    {
    case BM_None:
      break;
    case BM_North:
      if (wraparound)
	pos.y = (pos.y+1) % PPSharedParam::instance()->getYSize();
      else
	pos.y = min(pos.y+1, PPSharedParam::instance()->getYSize() - 1);
      break;
    case BM_South:
      if (wraparound)
	pos.y = (pos.y-1+PPSharedParam::instance()->getYSize()) % PPSharedParam::instance()->getYSize();
      else
	pos.y = max(pos.y-1, 0);
      break;
    case BM_East:
      if (wraparound)
	pos.x = (pos.x+1) % PPSharedParam::instance()->getXSize();
      else
	pos.x = min(pos.x+1, PPSharedParam::instance()->getXSize() - 1);
      break;
    case BM_West:
      if (wraparound)
	pos.x = (pos.x-1+PPSharedParam::instance()->getXSize()) % PPSharedParam::instance()->getXSize();
      else
	pos.x = max(pos.x-1, 0);
      break;
    default:
      errorlog << "What is move? " << m << ende;
      break;
    }

  v[idx] = pos;
}

/***************************************************************************************/

bool
State::isCapturePred(int pred_idx) const
{
  for (int i=0; i<PPSharedParam::instance()->getNumPrey(); i++)
    if (isCapture(pred_idx, i))
      return true;
  return false;
}

bool
State::isCapturePrey(int prey_idx) const
{
  for (int i=0; i<PPSharedParam::instance()->getNumPred(); i++)
    if (isCapture(i, prey_idx))
      return true;
  return false;
}

bool
State::isCaptureAny() const
{
  for (int i=0; i<PPSharedParam::instance()->getNumPred(); i++)
    if (isCapturePred(i))
      return true;
  return false;
}


/***************************************************************************************/

std::ostream&
operator<<(std::ostream& o, const State& s)
{
  o << "State(";
  o << "pred:";
  for (vector<Location>::const_iterator iter = s.vPredLoc.begin();
       iter != s.vPredLoc.end();
       ++iter)
    {
      o << *iter;
    }
  o << ", prey:";
  for (vector<Location>::const_iterator iter = s.vPreyLoc.begin();
       iter != s.vPreyLoc.end();
       ++iter)
    {
      o << *iter;
    }
  o << ")";
  return o;
}

//example sense:(pred (0 2 2) ) (prey (1 2 2) )
std::istream&
operator>>(std::istream& is, State& s)
{
  char ch;
  string str;

  try
    {
      //the predators
      if (!skip_white_space(is))
	throw State::ParseException();

      is.get(ch);
      if (ch != '(')
	throw State::ParseException();

      is >> str;

      if (str != "pred")
	throw State::ParseException();

      if (!State::readLocs(is, s.vPredLoc))
	throw State::ParseException();
	
      is.get(ch);
      if (ch != ')')
	throw State::ParseException();

      //the prey
      if (!skip_white_space(is))
	throw State::ParseException();

      is.get(ch);
      if (ch != '(')
	throw State::ParseException();

      is >> str;

      if (str != "prey")
	throw State::ParseException();

      if (!State::readLocs(is, s.vPreyLoc))
	throw State::ParseException();
	
      is.get(ch);
      if (ch != ')')
	throw State::ParseException();
    }
  catch ( State::ParseException E )
    {
      is.setstate(ios::failbit);
    }

  return is;
}

//static method
bool
State::readLocs(std::istream& is, std::vector<Location> &v)
{
  char ch;
  string str;
  int num;
  Location l;
  
  v.clear();
  
  try
    {
      for (;;)
	{

	  if (!skip_white_space(is))
	    throw State::ParseException();

	  is.get(ch);
	  if (!is)
	    break; //we're done finding locations!
	  if (ch != '(')
	    {
	      is.putback(ch);
	      break;
	    }
	  
	  is >> num >> l;

	  if (!is)
	    throw State::ParseException();

	  v.push_back(l);
	  
	  if (!skip_white_space(is))
	    throw State::ParseException();
	
	  is.get(ch);
	  if (ch != ')')
	    throw State::ParseException();
	  
	}
      
    }
  catch ( State::ParseException E )
    {
      return false;
    }

  return true;
}


/********************************************************************************************/
unsigned int State::s_num_states = 0;
bool State::s_num_states_calc = false;

//static method
unsigned int
State::calcNumStates()
{
  if (PPSharedParam::instance() == NULL)
    errorlog << "Tried to calculate num states before parameter initialization!" << ende;
  return int_pow(PPSharedParam::instance()->getTotalSize(), PPSharedParam::instance()->getNumPred() + PPSharedParam::instance()->getNumPrey());
}

unsigned int
State::getNumStates()
{
  if (!s_num_states_calc)
    {
      s_num_states = calcNumStates();
      s_num_states_calc = true;
    }
  return s_num_states;
}  
