/* A class decribing the state of the world */

#ifndef _STATE_H_
#define _STATE_H_

#include <vector>
#include "PPSharedTypes.hpp"
#include "StateFilter.hpp"

/* This will use the global params in PPSharedParams about
   x_size, y_size, num_pred, num_prey */

class State
{
public:
  State() {}
  State(const State& s) : vPredLoc(s.vPredLoc), vPreyLoc(s.vPreyLoc) {}
  State(unsigned int flatval) { setFromFlatVal(flatval); }
  ~State() {}

  void clear() { vPredLoc.clear(); vPreyLoc.clear(); }

  //this will be relative to the num_pred, num_prey in params, even if there aren't
  //that many listed
  unsigned int flatVal() const;
  void setFromFlatVal(unsigned flatval);

  void applyBasicMovePred(basic_move_t m, int idx, bool wraparound)
  { applyBasicMove(vPredLoc, m, idx, wraparound); }
  void applyBasicMovePrey(basic_move_t m, int idx, bool wraparound)
  { applyBasicMove(vPreyLoc, m, idx, wraparound); }

  bool isCapture(int pred_idx, int prey_idx) const
  { return vPredLoc[pred_idx] == vPreyLoc[prey_idx]; }
  //returns true if this predator captured someone
  bool isCapturePred(int pred_idx) const;
  //returns true is this prey was captured
  bool isCapturePrey(int prey_idx) const;
  // whether there is any capture here
  bool isCaptureAny() const;  
  
  static unsigned int getNumStates();

  friend std::ostream& operator<<(std::ostream& o, const State& s);
  friend std::istream& operator>>(std::istream& i, State& s);

private:
  std::vector<Location> vPredLoc;
  std::vector<Location> vPreyLoc;

  void applyBasicMove(std::vector<Location>& v, basic_move_t m, int idx, bool wraparound);
  
  //This actually read in triples, and throws away the first number
  static bool readLocs(std::istream& is, std::vector<Location> &v);
  
  class ParseException
  {
  public:
    ParseException() {}
    ~ParseException() {}
  };

  //static vars
  static unsigned int s_num_states;
  static bool s_num_states_calc;
  static unsigned int calcNumStates();
  
};

// Filters on whether the state is a capture
class CaptureStateFilter
  : public StateFilter
{
public:
  CaptureStateFilter(bool use_captures) : StateFilter(), use_captures(use_captures) {}
  ~CaptureStateFilter() {}

  bool useState(unsigned state_idx) const
  { return State(state_idx).isCaptureAny() ? use_captures : !use_captures; }

private:
  // true if we want to return true on cpature states.
  bool use_captures;
};

#endif
