/* This is an act event to send a communication to another agent */

#include "CommActEvent.hpp"
#include "Logger.hpp"
#include "CreateCommSenseEvent.hpp"
#include "PredPreyWorld.hpp"
#include "SimEngine.hpp"

using namespace spades;

CommActEvent::CommActEvent (spades::SimTime t, spades::AgentID a, entityid_t sender,
			    const char* m, unsigned mlen)
  : ActEvent(t, a), sender(sender)
{
  msg = new char[mlen+1];
  memcpy(msg, m, mlen);
  msg[mlen] = 0;
}

CommActEvent::~CommActEvent ()
{
  if (msg) delete msg;
}


void
CommActEvent::addReceiver(spades::AgentID a)
{
  vReceivers.push_back(a);
}


void
CommActEvent::print (std::ostream & o) const
{
  o << "CommActEvent(t:" << getTime()
    << ",a:" << getAgent()
    << ",msg:" << msg
    << ",rec:";
  for (recv_list_t::const_iterator iter = vReceivers.begin();
       iter != vReceivers.end();
       iter++)
    {
      o << *iter << ' ';
    }
  o << ")";
}

bool
CommActEvent::realizeEventWorldModel(spades::WorldModel* pWM)
{
  PredPreyWorld* p = (PredPreyWorld*)pWM;
  if (vReceivers.empty())
    errorlog << *this << ": realizing with no receivers!" << ende;
  for (recv_list_t::const_iterator iter = vReceivers.begin();
       iter != vReceivers.end();
       iter++)
    {
      p->pSimEngine->enqueueEvent(new CreateCommSenseEvent(getTime()+1, *iter, sender, msg));
    }
  return true;
}

