/* Create a sense event which is the full state of the world */

#include <strstream>
#include "SimEngine.hpp"

#include "CreateFullSenseEvent.hpp"
#include "SenseEvent.hpp"
#include "PredPreyWorld.hpp"
#include "Logger.hpp"

using namespace spades;
using namespace std;

CreateFullSenseEvent::CreateFullSenseEvent (spades::SimTime t, spades::AgentID a)
  : CreateSenseEvent(t, a)
{
}


void
CreateFullSenseEvent::print (std::ostream & o) const
{
  o << "CreateFullSenseEvent(" << getTime () << ", " << getAgent() << ")";
}



bool
CreateFullSenseEvent::realizeEventWorldModel(spades::WorldModel* pWM)
{
  PredPreyWorld* pPP = (PredPreyWorld*)pWM;
  //Queue up the next CreateNormSenseEvent
  CreateFullSenseEvent* pE =
    new CreateFullSenseEvent(getTime() + pPP->getSenseInterval(), getAgent());
  pPP->pSimEngine->enqueueEvent(pE);
  return true;
}


//from CreateSenseEvent
spades::SenseEvent*
CreateFullSenseEvent::createSense(spades::WorldModel* p)
{
  PredPreyWorld* pPP = (PredPreyWorld*)p;

  //No sense latency!
  SimTime arrive_time = getTime();
  
  actionlog(150) << "Creating a FullSenseEvent"
		 << "(ts:" << getTime()
		 << ", ta:" << arrive_time
		 << ", a: " << getAgent()
		 << ")"
		 << ende;

  ostrstream o;
  o << "(sense ";

  o << "(pred ";
  for (PredPreyWorld::map_pred_t::iterator iter = pPP->map_pred.begin();
       iter != pPP->map_pred.end();
       ++iter)
    {
      iter->second.printCompact(o);
      o << ' ';
    }
  o << ") ";

  o << "(prey ";
  for (PredPreyWorld::map_prey_t::iterator iter = pPP->map_prey.begin();
       iter != pPP->map_prey.end();
       ++iter)
    {
      iter->second.printCompact(o);
      o << ' ';
    }
  o << ") ";

  o << ") ";

  o << ends;

  SenseEvent* pE =
    new SenseEvent(TT_Regular, getTime(), arrive_time, getAgent());

  DataArray d(o);
  pE->setData(d);
  
  //note that we do not unfreeze ostrstream since DataArray took it

  return pE;
}


