/* this defines the main act event for our agents, a velocity change */

#include "MoveActEvent.hpp"
#include "Logger.hpp"
#include "PredPreyWorld.hpp"
#include "CreateActionInformEvent.hpp"
#include "SimEngine.hpp"

using namespace spades;

MoveActEvent::MoveActEvent (spades::SimTime t, spades::AgentID a, basic_move_t m)
  : ActEvent(t, a), move(m)
{
}


void
MoveActEvent::print (std::ostream & o) const
{
  o << "MoveActEvent(t:" << getTime() << ", a:" << getAgent() << ", m:" << move << ")";
}

bool
MoveActEvent::realizeEventWorldModel(spades::WorldModel* pWM)
{
  PredPreyWorld* p = (PredPreyWorld*)pWM;

  PredPreyWorld::map_pred_t::iterator prediter = p->map_pred.find(getAgent());
  if (prediter != p->map_pred.end())
    {
      actionlog(90) << *this << ": realizing for pred " << prediter->second << ende;
      prediter->second.setNextMove(move);
      if (PredPreyParam::instance()->getTellCoachAgentActions() && p->pPredCoach)
	p->pSimEngine->enqueueEvent(new CreateActionInformEvent(getTime(), p->pPredCoach->getAgentID(),
								prediter->second.getID(),
								move));
      return true;
    }
  
  PredPreyWorld::map_prey_t::iterator preyiter = p->map_prey.find(getAgent());
  if (preyiter != p->map_prey.end())
    {
      actionlog(90) << *this << ": realizing for prey " << preyiter->second << ende;
      preyiter->second.setNextMove(move);
      if (PredPreyParam::instance()->getTellCoachAgentActions() && p->pPreyCoach)
	p->pSimEngine->enqueueEvent(new CreateActionInformEvent(getTime(), p->pPreyCoach->getAgentID(),
								preyiter->second.getID(),
								move));
      return true;
    }

  errorlog << *this << ": I can't find the agent!" << ende;

  return false;
}


