/* -*- Mode: C++ -*- */

#ifndef _PREDPREYPARAM_H_
#define _PREDPREYPARAM_H_

#include "EngineParam.hpp"

class PredPreyParam
   : public spades::EngineParam
{
public:

  PredPreyParam (const int &argc, const char *const *argv);
  ~PredPreyParam ();

  static const PredPreyParam* instance() { return s_instance; }
  
private:
  static PredPreyParam* s_instance;
  
protected:
  //virtual void writeConfig ( ostream& o );

  void addAll2Maps ();
  void setDefaultValues ();
  void postReadProcessing();

public:

  int getXSize() const { return x_size; }
  int getYSize() const { return y_size; }
  int getTotalSize() const { return total_size; } // = x_size * y_size
  
  int getDiscreteStepSize() const { return  discrete_step_size; }
  
  int getSimulationLength() const { return simulation_length; }
  int getNumCommServersWanted() const { return num_comm_servers_wanted; }

  std::string getPredName() const { return pred_name; }
  std::string getPreyName() const { return prey_name; }
  std::string getPredCoachName() const { return pred_coach_name; }
  std::string getPreyCoachName() const { return prey_coach_name; }

  int getNumPred() const { return num_pred; }
  int getNumPrey() const { return num_prey; }
  bool getUsePredCoach() const { return use_pred_coach; }
  bool getUsePreyCoach() const { return use_prey_coach; }
  
  int getReward() const { return reward; }

  bool getFixPreyLocations() const { return fix_prey_locations; }

  bool getWraparound() const { return wraparound; }

  bool getTellCoachAgentActions() const { return tell_coach_agent_actions; }

protected:

  int x_size;
  int y_size;

  int total_size;

  int discrete_step_size;
  
  int simulation_length;
  int num_comm_servers_wanted;

  std::string pred_name;
  std::string prey_name;
  std::string pred_coach_name;
  std::string prey_coach_name;

  int num_pred;
  int num_prey;
  bool use_pred_coach;
  bool use_prey_coach;
  
  int reward;

  bool fix_prey_locations;

  bool wraparound;

  bool tell_coach_agent_actions;

};


#endif
