/* This defines the world model for a simple non-colliding ball world */

#ifndef _PREDPREYWORLD_H_
#define _PREDPREYWORLD_H_

#include <map>
#include "WorldModel.hpp"
#include "PredPreyParam.hpp"
#include "Entity.hpp"
#include "CoachEntity.hpp"
class CommActEvent;

class PredPreyWorld
  : public spades::WorldModel
{
public:
  PredPreyWorld ();
  ~PredPreyWorld ();

  //These are all the WorldModel functions
  spades::EngineParam *parseParameters (const int argc, const char *const *argv);

  /* Most of the real initialization/finalization work should be done here */
  bool initialize (spades::SimEngine * pSE);
  bool finalize ();

  void pauseModeCallback ();

  spades::SimTime simToTime (spades::SimTime time_curr, spades::SimTime time_desired);

  spades::DataArray getMonitorHeaderInfo ();
  spades::DataArray getMonitorInfo (spades::SimTime time);
  void parseMonitorMessage (const char* data, unsigned datalen);

  spades::SimTime getMinActionLatency() const { return 1; }
  spades::ActEvent* parseAct(spades::SimTime t, spades::AgentID a,
			     const char* data, unsigned datalen) const;

  spades::SimTime getMinSenseLatency() const { return 0; }

  bool agentConnect (spades::AgentID agent, spades::AgentTypeDB::AgentTypeConstIterator at);
  bool agentDisappear (spades::AgentID agent, spades::AgentLostReason reason);

  void notifyCommserverConnect (spades::ServerID s) {}
  void notifyCommserverDisconnect (spades::ServerID s) {}

  //Now our own functions
  spades::SimTime getSenseInterval();
  
protected:

  //returns number of reward signals sent
  //can queue up reward sending events
  int checkReward(spades::SimTime time);

  void logPositions(int level);

  entityid_t getEntityIDFromAgentID(spades::AgentID a, pp_agent_type_t *ptype = NULL) const;

  CommActEvent* createCommEvent(spades::SimTime t, spades::AgentID a, entityid_t eid,
				std::istream& in) const;
  
  spades::SimEngine * pSimEngine;

  entityid_t next_entity_id;
  
  typedef std::map<spades::AgentID, Entity> map_pred_t;
  typedef std::map<spades::AgentID, Entity> map_prey_t;

  spades::AgentTypeDB::AgentTypeConstIterator pred_type;
  spades::AgentTypeDB::AgentTypeConstIterator prey_type;
  spades::AgentTypeDB::AgentTypeConstIterator pred_coach_type;
  spades::AgentTypeDB::AgentTypeConstIterator prey_coach_type;

  map_pred_t map_pred;
  map_prey_t map_prey;
  CoachEntity* pPredCoach;
  CoachEntity* pPreyCoach;

  //Now all the PPEvents which can affect us
  friend class CreateFullSenseEvent;
  friend class MoveActEvent;
  friend class CommActEvent;
  friend class ReqShutdownActEvent;
  
};


#endif
