/* A behavior to execute q-learning with coach involvement */

#include "QLearnWCoachBehavior.hpp"
#include "Logger.hpp"
#include "State.hpp"
#include "Policy.hpp"
#include "random.hpp"

using namespace std;
using namespace spades;
using namespace spadesalib;

QLearnWCoachBehavior::QLearnWCoachBehavior(AgentCommInterface* pacs,
					   float gamma,
					   explore_t exp, float alpha_explore, float boltzmann_constant,
					   float coach_prob,
					   bool random_on_advised_disable,
					   const char* disable_map_fn)
  : BaseQLearnBehavior(pacs, gamma, exp, alpha_explore, boltzmann_constant, disable_map_fn),
    coach_prob(coach_prob), random_on_advised_disable(random_on_advised_disable)
{
}


QLearnWCoachBehavior::~QLearnWCoachBehavior()
{
}

  
void
QLearnWCoachBehavior::respondSenseQLearn(Behavior::SenseType st)
{
  switch (st)
    {
    case ST_None:
      errorlog << "I don't know what to do with ST_None" << ende;
      break;
    case ST_TimeNotify:
      //do nothing, but we should never get here
      break;
    case ST_State:
      //we do need to update from last time, because some entries will get cleared
      updateQValue();
      if (getVecCapturedAgents().empty())
	{
	  //just request from the coach, we'll take actions once we hear back from the coach
	  sendCoachRequest();
	}
      else
	{
	  //on a capture, we just do nothing, we're going to be moved
	  sendMove(BM_None);
	  setLastAction(BM_None);
	}
      stateSenseUpdate();
      break;
    case ST_Comm:
      if (getCommType() == CT_ActionAdvice)
	{
	  basic_move_t action;
	  unsigned state_idx = getCurrState().flatVal();
	  if (prob_random() < coach_prob)
	    {
	      if (qtable.isEnabled(state_idx, getCommActionAdvice()))
		{
		  actionlog(20) << "QLearnWCoachBehavior: " << state_idx
				<< " I am obeying coach: "
				<< getCommActionAdvice() << ende;
		  action = getCommActionAdvice();
		}
	      else
		{
		  actionlog(20) << "QLearnWCoachBehavior: " << state_idx
				<< " can't follow coach advice!: "
				<< getCommActionAdvice() << ende;
		  if (random_on_advised_disable)
		    action = (basic_move_t)qtable.getActRandom(state_idx);
		  else
		    action = (basic_move_t)qtable.getActFirst(state_idx);		    
		}
	    }
	  else
	    {
	      actionlog(20) << "QLearnWCoachBehavior: using the prevailing exploitation/exploration mode: "
			    << exp_type << "(coach: " << getCommActionAdvice() << ")" << ende;
	      action = (basic_move_t)qtable.getAct(exp_type, state_idx, alpha_explore, boltzmann_constant);
	    }
	  actionlog(20) << "QLearnWCoachBehavior: sending action: " << action << ende;
	  sendMove(action);
	  setLastAction(action);
	}
      else
	{
	  errorlog << "I only handle comm of action advice: " << getCommType() << ende;
	}
      break;
    default:
      errorlog << "I don't understand st: " << st << ende;
      break;
    }
  
}


bool
QLearnWCoachBehavior::initialize()
{
  return true;
}


